/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * Authors: Travis Reitter <treitter@gmail.com>
 */

#include <config.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <hildon/hildon.h>

#include "milk-list-store.h"

static void
milk_list_store_tree_model_init (GtkTreeModelIface *iface);

G_DEFINE_TYPE_EXTENDED (MilkListStore,
                        milk_list_store,
                        G_TYPE_OBJECT,
                        0,
                        G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
                                               milk_list_store_tree_model_init));

#define MILK_LIST_STORE_PRIVATE(o) \
                (G_TYPE_INSTANCE_GET_PRIVATE ((o), MILK_TYPE_LIST_STORE, MilkListStorePrivate))

#define MILK_LIST_STORE_STAMP_INVALID 0

struct _MilkListStorePrivate
{
        gint stamp;
        gint node_count;
};

static GtkTreeModelFlags
milk_list_store_get_flags (GtkTreeModel *model)
{
        return GTK_TREE_MODEL_LIST_ONLY;
}

static gint
milk_list_store_get_n_columns (GtkTreeModel *model)
{
        return MILK_LIST_STORE_COLUMN_LAST;
}

static GType
milk_list_store_get_column_type (GtkTreeModel *model,
                                 gint          column)
{
        switch (column) {
                case MILK_LIST_STORE_COLUMN_PRIORITY:
                        return G_TYPE_UINT;

                case MILK_LIST_STORE_COLUMN_TASK:
                        /* FIXME: define the MilkTask class and use
                         * MILK_TYPE_TASK here */
                        return G_TYPE_STRING;

                default:
                        g_warning (G_STRLOC ": invalid column: %d", column);
                        return G_TYPE_INVALID;
        }
}

static gboolean
milk_list_store_get_iter (GtkTreeModel *model,
                          GtkTreeIter  *iter,
                          GtkTreePath  *path)
{
        g_return_val_if_fail (MILK_IS_LIST_STORE (model), FALSE);
        g_return_val_if_fail (iter, FALSE);
        g_return_val_if_fail (gtk_tree_path_get_depth (path) == 1, FALSE);

        /* FIXME: implement this */
        g_warning (G_STRLOC ": FIXME: not implemented");

        return TRUE;
}

static GtkTreePath*
milk_list_store_get_path (GtkTreeModel *model,
                          GtkTreeIter  *iter)
{
        g_return_val_if_fail (MILK_IS_LIST_STORE (model), NULL);
        g_return_val_if_fail (iter, NULL);

        /* FIXME: implement this */
        g_warning (G_STRLOC ": FIXME: not implemented");

        return NULL;
}

static void
milk_list_store_get_value (GtkTreeModel *model,
                           GtkTreeIter  *iter,
                           gint          column,
                           GValue       *value)
{
        g_return_if_fail (MILK_IS_LIST_STORE (model));
        g_return_if_fail (iter);
        g_return_if_fail (column < 0);
        g_return_if_fail (value);

        /* FIXME: implement this */
        g_warning (G_STRLOC ": FIXME: not implemented");
}

static gboolean
milk_list_store_iter_next (GtkTreeModel *model,
                           GtkTreeIter  *iter)
{
        g_return_val_if_fail (MILK_IS_LIST_STORE (model), FALSE);
        g_return_val_if_fail (iter, FALSE);

        /* FIXME: implement this */
        g_warning (G_STRLOC ": FIXME: not implemented");

        return TRUE;
}

static gboolean
milk_list_store_iter_nth_child (GtkTreeModel *model,
                                GtkTreeIter  *iter,
                                GtkTreeIter  *parent,
                                gint          index)
{
        MilkListStorePrivate *priv = MILK_LIST_STORE_PRIVATE (model);

        g_return_val_if_fail (MILK_IS_LIST_STORE (model), FALSE);
        g_return_val_if_fail (iter, FALSE);
        /* we're one-dimensional */
        g_return_val_if_fail (!parent, FALSE);
        g_return_val_if_fail (index < 0, FALSE);
        g_return_val_if_fail (index >= priv->node_count, FALSE);

        /* FIXME: implement this */
        g_warning (G_STRLOC ": FIXME: not implemented");

        return TRUE;
}

static gboolean
milk_list_store_iter_children (GtkTreeModel *model,
                               GtkTreeIter  *iter,
                               GtkTreeIter  *parent)
{
        /* we're one-dimensional, so this is a degenerate case */
        return milk_list_store_iter_nth_child (model, iter, parent, 0);
}

static gboolean
milk_list_store_iter_has_child (GtkTreeModel *model,
                                GtkTreeIter  *iter)
{
        g_return_val_if_fail (MILK_IS_LIST_STORE (model), FALSE);
        g_return_val_if_fail (iter, FALSE);

        return FALSE;
}

static gint
milk_list_store_iter_n_children (GtkTreeModel *model,
                                 GtkTreeIter  *iter)
{
        MilkListStorePrivate *priv = MILK_LIST_STORE_PRIVATE (model);

        g_return_val_if_fail (MILK_IS_LIST_STORE (model), -1);
        g_return_val_if_fail (iter, -1);

        /* we're one-dimensional */
        if (iter)
                return 0;

        return priv->node_count;
}

static gboolean
milk_list_store_iter_parent (GtkTreeModel *model,
                             GtkTreeIter  *iter,
                             GtkTreeIter  *child)
{
        g_return_val_if_fail (MILK_IS_LIST_STORE (model), FALSE);
        g_return_val_if_fail (iter, FALSE);
        g_return_val_if_fail (child, FALSE);

        return FALSE;
}

static void
milk_list_store_get_property (GObject    *object,
                               guint       property_id,
                               GValue     *value,
                               GParamSpec *pspec)
{
        MilkListStorePrivate *priv = MILK_LIST_STORE_PRIVATE (object);
        switch (property_id)
        {
                default:
                        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id,
                                        pspec);
        }
}

static void
milk_list_store_set_property (GObject      *object,
                               guint         property_id,
                               const GValue *value,
                               GParamSpec   *pspec)
{
        MilkListStorePrivate *priv = MILK_LIST_STORE_PRIVATE (object);
        switch (property_id)
        {
                default:
                        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id,
                                        pspec);
        }
}

static void
milk_list_store_dispose (GObject *object)
{
        MilkListStorePrivate *priv = MILK_LIST_STORE_PRIVATE (object);

        G_OBJECT_CLASS (milk_list_store_parent_class)->dispose (object);
}

static void
milk_list_store_constructed (GObject* object)
{
        MilkListStore *self = MILK_LIST_STORE (object);
        MilkListStorePrivate *priv = MILK_LIST_STORE_PRIVATE (object);

        priv->stamp = MILK_LIST_STORE_STAMP_INVALID + 1;
}

static void
milk_list_store_class_init (MilkListStoreClass *klass)
{
        GObjectClass *object_class = G_OBJECT_CLASS (klass);

        g_type_class_add_private (klass, sizeof (MilkListStorePrivate));

        object_class->get_property = milk_list_store_get_property;
        object_class->set_property = milk_list_store_set_property;
        object_class->constructed = milk_list_store_constructed;
        object_class->dispose = milk_list_store_dispose;

        /* FIXME: trigger the signals with gtk_tree_model_row_inserted(), etc.
         */
}

static void
milk_list_store_init (MilkListStore *self)
{
        self->priv = MILK_LIST_STORE_PRIVATE (self);
}

static void
milk_list_store_tree_model_init (GtkTreeModelIface *iface)
{
        iface->get_flags       = milk_list_store_get_flags;
        iface->get_n_columns   = milk_list_store_get_n_columns;
        iface->get_column_type = milk_list_store_get_column_type;
        iface->get_iter        = milk_list_store_get_iter;
        iface->get_path        = milk_list_store_get_path;
        iface->get_value       = milk_list_store_get_value;
        iface->iter_next       = milk_list_store_iter_next;
        iface->iter_children   = milk_list_store_iter_children;
        iface->iter_has_child  = milk_list_store_iter_has_child;
        iface->iter_n_children = milk_list_store_iter_n_children;
        iface->iter_nth_child  = milk_list_store_iter_nth_child;
        iface->iter_parent     = milk_list_store_iter_parent;
}

MilkListStore*
milk_list_store_new ()
{
        return g_object_new (MILK_TYPE_LIST_STORE,
                             NULL);
}
